import re,string
import os,sys
import glob

class apsis_catalogs:

    """Reads in the apsis catalogs and stores the information"""
    
    def __init__(self, path, verb=None):

        if verb:
            print >>sys.stderr, "\r Initializing class "
        
        # Set up the standard apsis output file names
        self.path     = path
        self.verb     = verb
        self.detcat   = os.path.join(self.path,"Catalogs","detectionImage.cat")
        self.bpzcat   = os.path.join(self.path,"Catalogs","bpz.cat")
        self.colorcat = os.path.join(self.path,"Catalogs","multicolor.cat")
        self._get_names()
        self._read_detection()
        self._read_bpz()
        self._read_multicolor()
        
        return

    def _get_names(self):

        self.sci_images = glob.glob(os.path.join(self.path,'Images','*_drz_sci.fits'))
        self.wgt_images = glob.glob(os.path.join(self.path,'Images','*_drz_sci_weight.fits'))
        self.seg_image  = os.path.join(self.path,'Images','detectionImage_SEGM.fits')

        self.sci_images.sort()
        self.wgt_images.sort()

        self.sci_image = {}
        self.wgt_image = {}

        for i in range(len(self.sci_images)):
            
            vals   = self.sci_images[i].split('_')
            filter = vals[1]
            self.sci_image[filter] = self.sci_images[i]

            vals   = self.wgt_images[i].split('_')
            filter = vals[1]
            self.wgt_image[filter] = self.wgt_images[i]

            #print "filter ",filter
            #print filter,self.sci_image[filter]
            #print filter,self.wgt_image[filter]

        return

    
    def _read_detection(self):
        
        if self.verb:
            print >>sys.stderr, "\r Reading:",self.detcat
	    
        self.THETA = {}
        self.X_IMAGE = {}
        self.Y_IMAGE = {}
        self.A_IMAGE = {}
        self.B_IMAGE = {}
        self.KRON_RADIUS = {}
        self.RADIUS = {}
        self.SIZE = {}
        self.BACK = {}
        self.THRESH = {}
        self.FLAG = {}
        self.RMS = {}
        self.CLASS_STAR = {}
        
        for line in open(self.detcat).readlines():
            
            vals = line.split()
            
            if vals[0][0] == "#":
                continue
            
            ID = str(vals[0])
            self.X_IMAGE[ID]     = float(vals[1])
            self.Y_IMAGE[ID]     = float(vals[2])
            self.KRON_RADIUS[ID] = float(vals[23])
            self.A_IMAGE[ID]     = float(vals[38])*self.KRON_RADIUS[ID]
            self.B_IMAGE[ID]     = float(vals[39])*self.KRON_RADIUS[ID]
            self.THETA[ID]       = float(vals[42])
            self.RADIUS[ID]      = 1.5*(self.A_IMAGE[ID])
            self.SIZE[ID]        = int(2*self.RADIUS[ID])
            self.BACK[ID]        = float(vals[24])
            self.THRESH[ID]      = float(vals[25])
            self.FLAG[ID]        = int(vals[67])
            self.RMS[ID]         = self.THRESH[ID]/1.5
            self.CLASS_STAR[ID]  = float(vals[68])
            
        return

    def _read_bpz(self):
        
        if self.verb:
            print >>sys.stderr, "\r Reading:",self.bpzcat

        self.Z_B     = {}
        self.Z_B_MIN = {}
        self.Z_B_MAX = {}
        self.T_B     = {}
        self.ODDS    = {}
        self.Z_ML    = {}
        self.T_ML    = {}
        self.CHI2    = {}
        
        for line in open(self.bpzcat).readlines():
		    
            vals = line.split()
		    
            if vals[0][0] == "#":
                continue
		    
            ID           = str(vals[0])
            self.Z_B[ID]     = float(vals[1])
            self.Z_B_MIN[ID] = float(vals[2])
            self.Z_B_MAX[ID] = float(vals[3])
            self.T_B[ID]     = float(vals[4])
            self.ODDS[ID]    = float(vals[5])
            self.Z_ML[ID]    = float(vals[6])
            self.T_ML[ID]    = float(vals[7])
            self.CHI2[ID]    = float(vals[8])
		    
        return



    def _read_multicolor(self):

        if self.verb:
            print >>sys.stderr, "\r Reading:",self.colorcat
            
        # Make a clean dictionarys and lists
        idx     = {}
        col     = []
        name    = []
        filters = []
        
        # Read in the header's comments
        for line in open(self.colorcat).readlines():
            vals = line.split()
        
            if vals[0] == "#":
                col.append(vals[1])
                name.append(vals[2])
            else:
                continue
        
        # Go through the list of comments stored
        for i in range(len(col)):

            # Entries are of the kind: HST_ACS_WFC_F775W_MAG_FINAL
            if re.match("HST_ACS_WFC",name[i]):
                vals = name[i].split('_')
                FILTER = vals[3]
                filter = string.swapcase(vals[3])
                
                #Extract index for MAG_FINAL
                if vals[4] == 'MAG':
                    idx[filter] = {}
                    filters.append(filter)
                    idx[filter]['MAG'] = int(col[i]) - 1
                    
                #Extract index for MAGERR_FINAL
                elif vals[4] == "MAGERR":  
                    idx[filter]['ERR'] = int(col[i]) - 1
                    
                    
        self.filters = filters
        # Create the dictionaries for each filter
        self.MAG = {}
        self.ERR = {}
        self.GOOD = {}
        for filter in self.filters:
            self.MAG[filter] = {}
            self.ERR[filter] = {}
            #i1 = idx[filter]['MAG']
            #i2 = idx[filter]['ERR']
            #print "%6s %3d %3d" % (filter,i1+1,i2+1)

        # Open the color catalog and read it
        for line in open(self.colorcat).readlines():

            vals = line.split()
            if vals[0][0] == "#":
                continue

            ID = str(vals[0])
            self.GOOD[ID] = 1
            # Go trough the filters
            for filter in self.filters:
                i1 = idx[filter]['MAG']
                i2 = idx[filter]['ERR']
                self.MAG[filter][ID] = float(vals[i1])
                self.ERR[filter][ID] = float(vals[i2])

                # Check that it's in all exposures
                if self.MAG[filter][ID] == -99:
                    self.GOOD[ID] = 0

        return


