import os,string,popen2,re
import tableio,numarray,math

########################################################
# Colecction of tools to inteact with ds9 from python
#
# F. Menanteau JHU.
#


def prepare(nf, # Number of fields
            cmap     = 'Heat',
            scale    = 'linear',
            scalemode= '99.0',
            tile     = 'yes',
            tilemode = 'column', # column,grid or row
            verb     = None):
            
    """ Start up ds9 in XPA mode and several options """


    # Check env variables and set to proper values
    check_env(verb=verb);
    # Check if DS9 is already running
    check_ds9(verb=verb);
    # Start reseting
    os.system("xpaset -p ds9 scale "     + scale)
    os.system("xpaset -p ds9 scale mode" + scalemode)
    os.system("xpaset -p ds9 cmap "      + cmap)
    os.system("xpaset -p ds9 frame delete all")
    # Create the frames
    for i in range(nf):
        os.system("xpaset -p ds9 frame new")
        os.system("xpaset -p ds9 scale mode" + scalemode)
    # Tile up
    if tile:
        os.system ("xpaset -p ds9 tile ")
        os.system ("xpaset -p ds9 tile " + tilemode)

    return


def ellipse(shape,color='green',text='',options='',units='deg'):

    """ Draw an ellipse of shape=(x,y,a,b,theta) on ds9 """

    if units == 'rad':
        theta  = shape[4]*180.0/math.pi
    else:
        theta  = shape[4]

    format = "image; ELLIPSE(%8.2f,%8.2f,%7.2f,%7.2f,%6.1f) # color=%s text={%s} %s\n";
    reg = os.popen('xpaset ds9 regions', 'w')
    reg.write(format % (shape[0],shape[1],shape[2],shape[3],theta,color,text,options))
    reg.flush()
    reg.close()
    return


def cross(x,y,color='green',text='',options=''):

    """ Draw an point cross at x,y on ds9 """

    format = "physical;point(%8.2f,%8.2f) # point=x color=%s text={%s} %s\n"
    reg = os.popen('xpaset ds9 regions', 'w')
    reg.write(format % (x,y,color,text,options))
    reg.flush()
    reg.close()
    return


def frame(fr=1):
    ''' Go to frame fr'''
    os.system('xpaset -p ds9 frame ' + str(fr))
    return

def put(fitsfile,fr=None,zoom='yes',advance=None):
    """ Display fits image into ds9 """

    #check_env()
    #check_ds9()

    if fr:
        os.system("xpaset -p ds9 frame " + str(fr))

    os.system("xpaset -p ds9 file " + fitsfile)

    if zoom == 'yes':
        os.system("xpaset -p ds9 zoom to fit " + str(fr))

    if advance:
        os.system("xpaset -p ds9 frame next")

    return

def close():
    """ Close the ds9 session and window """
    print "## Closing ds9 ";
    os.system ("xpaset -p ds9 exit")
    print "## Bye ...";
    return

def check_env(var="XPA_METHOD",value="local",verb=None):

    """ Check for the XPA_METHOD env variable to be set """

    if os.environ.has_key(var):
        if verb: print "# %s is already set to: %s" % (var, os.environ[var])
    else:
        os.environ[var] = value
        if verb: print "# %s is NOW set to: %s" % (var,value)

def check_ds9(verb=None):

    ''' Check if ds9 is running in -xpa mode, if not we start it and
    wait until is ready to listen'''
    
    ans = get_xpaaccess()
    
    if ans == "yes":
        if verb: print "# ds9 -xpa is running OK"
        answer = ans
        
    if ans == "no":
        if verb: print "# ds9 xpa is *NOT* running... trying to start it now"
        ds9job = popen2.Popen4("ds9 -xpa local")

        answer = get_xpaaccess()
        i = 1
        while answer != "yes":
            #print "waiting...", i, answer
            answer = get_xpaaccess()
            i = i + 1

        if ds9job.poll() == -1:
            if verb: print "ds9 -xpa started OK"

    return answer


def get_xpaaccess():
    ''' To get the status of ds9 with xpa '''
    (stdout) = os.popen("xpaaccess ds9")
    ans = stdout.readline()
    ans = string.rstrip(ans)
    return ans


def inpath(program,verb=None):
    """ Checks if program is in the user's path """
    import os.path
    for path in os.environ['PATH'].split(':'):
        if os.path.exists( os.path.join(path,program) ):
            if verb: print "# program: %s found in: %s" % (program , os.path.join(path,program))
            return 1
    if verb: print "# program: %s NOT found in user's path " % program
    return 0



    
    

































#########################################################
# Read all the regions and reformat the new x,y points
# as new regions in the ds9 format
def read_and_parse_regions(counter=1):

    os.system("xpaset -p ds9 regions color blue")
    os.system("xpaset -p ds9 regions format ds9")
    (stdout,stdin) = popen2.popen2("xpaget ds9 regions")

    regexp_point = re.compile(r"(?P<type>physical;point)\("
                              r"(?P<x>[0-9]+),"
                              r"(?P<y>[0-9]+)\)\s*"
                              r"(?P<comments>#.+)?"
                              )
    x = []
    y = []
    i=0
    for line in stdout.readlines():
        
        # Check if point or polygon
        point = regexp_point.search(line)
        
        # Extract the information if a point was selected
        if point:
            type     = point.group('type')
            x.insert(i,point.group('x'))
            y.insert(i,point.group('y'))
            comments = point.group('comments')

            # Extract the comments and clean then up
            if (comments):
                comments = string.split(comments,"#")[1]
                comments = string.strip(comments)

            i = i + 1
            

    # After we've read all the lines and regions, we remove all the point
    # as regions are undeletable
    os.system("xpaset -p ds9 regions deleteall")

    # Format the coordinates and add comments
    comment = "color=blue text={region %d} delete = 0" % (counter)  
    regions = "physical;polygon(%s) # %s\n" % (pairs_into_list(x,y),comment)


    (stdout,stdin) = popen2.popen2("xpaset ds9 regions")
    #(io,w) = popen2.popen4("xpaset ds9 regions")
    stdin.write(regions)
    stdin.close()

# To split a list into x,y pairs
def split_into_pairs(list):
    i=0
    n=0
    x = []
    y = []
    while (i<len(list)):
        x.insert(n,float(list[i]))
        y.insert(n,float(list[i+1]))
        n = n + 1
        i = i + 2
    return x,y

# To split a list into x,y pairs
def pairs_into_list(x,y):
    i=0
    list = ""
    while (i<len(x)):
        if (i == 0):
            pair = "%s,%s"  % (x[i],y[i])
        else:
            pair = ",%s,%s" % (x[i],y[i])

        list = list + pair
        i = i + 1
    return list


# Parse the detection catalog and draw it into ds9
def parse_catalog(filename, color="red"):

    format = "image; ELLIPSE(%.2f,%.2f,%.2f,%.2f,%.1f) # %s\n"
    (stdout,stdin) = popen2.popen2("xpaset ds9 regions")

    comments = "# color=%s delete=0" % color

    file = open(filename,"r")
    for line in file.readlines():

        if line[0] == "#": continue

        vals = string.split(line)

        NUMBER  = int(vals[0])
        X_IMAGE = float(vals[1])
        Y_IMAGE = float(vals[2])
        A_IMAGE = float(vals[38])
        B_IMAGE = float(vals[39])
        THETA_IMAGE = float(vals[42])
        KRON_RADIUS = float(vals[23])

        region = format % (X_IMAGE,Y_IMAGE,A_IMAGE*KRON_RADIUS,B_IMAGE*KRON_RADIUS,THETA_IMAGE,comments)
        stdin.write(region)
    # close the writing units    
    file.close()
    stdin.close()

    
def parse_catalog_other(filename,color="red"):

    (X_IMAGE,Y_IMAGE,A_IMAGE,B_IMAGE,THETA_IMAGE,KRON_RADIUS) = tableio.get_data(filename,(1,2,38,39,42,23))
    regions = "/tmp/regions"
    hdr = "# filename ellipses regions"
    format = "image; ELLIPSE(%8.2f,%8.2f,%7.2f,%7.2f,%6.1f) # color=" + color
    tableio.put_data(regions,(X_IMAGE,Y_IMAGE,A_IMAGE*KRON_RADIUS,B_IMAGE*KRON_RADIUS,THETA_IMAGE),hdr,format)
    return regions
